<?php
namespace Teknisa\CRM\Controller;

use Zeedhi\Framework\DTO;
use PHPImap\Mailbox;
use Jenkins\Jenkins;
use Teknisa\Libs\Util\ConnectionFactory;
use Zeedhi\Framework\DataSource\DataSet;


class TicketBotController {
    
    protected $ticketBotService;
    
    /*Diretório de anexos (necessário para a lib PHPImap ler os anexos do email)*/
    protected $tempDirectoryPath;

    public function __construct(\Teknisa\CRM\Service\TicketBotService $ticketBotService, $environment, array $connectionParams, $session, $tempDirectoryPath) {
        $this->ticketBotService = $ticketBotService;
        $this->environment = $environment;
        $this->session = $session;
        $this->connectionParams = $connectionParams;
        $this->tempDirectoryPath = $tempDirectoryPath.'/attachs/';
    }
    
    public function connectToOrg(DTO\Request $request, DTO\Response $response, $nrorg){
        //Muda as configurações de conexão baseado na org
        $this->session->set('nrorg', $nrorg);
        ConnectionFactory::updateConnectionParams($this->connectionParams, null, null, $this->session);
        $this->getMailsAndInsertTickets($request, $response);
    }
    
    /**
     *  Inicia a rotina de captação de emails e criação de tickets 
    */
    public function getMailsAndInsertTickets(DTO\Request $row, DTO\Response $response){
        
        try {
            /**
             * set_time_limit(0);
             * ini_set('memory_limit', '-1'); 
            */
            $imapInfo = $this->ticketBotService->getImapInfo();
            //Faz a rotina pra todas as orgs que possuem a parametrização de IMAP
            foreach($imapInfo as $key => $imapInfoOrg){
                //Seta a Org como a org da parametrizacao atual
                $this->environment->setNrOrgTrab($key);
                $hostname = $imapInfoOrg['HOSTIMAP'];
                $username = $imapInfoOrg['USERIMAP'];
                $password = $imapInfoOrg['PASSIMAP'];
                
                $hostname = $this->buildHostname($hostname, 'INBOX');
                
                $mailBox = new Mailbox($hostname, $username, $password);
                
                $this->checkAttachDir();
                $mailBox->setAttachmentsDir($this->tempDirectoryPath);
                $mailsIds = $mailBox->searchMailbox('UNSEEN');

                if(!$mailsIds) {
                	$return = 'Mailbox is Empty.';
                }else{
                    $return = $this->ticketBotService->getMails($mailBox, $mailsIds);
                }
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('mails_ORG_'.$key, array($return)));
                var_dump($return);
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
    /**
     * Função para montar o hostname para o imap
     *  formato aceito pelo imap: {exemplo.imap.com:porta/options}PASTA.
     * 
     * @param string    $hostname           Endereço do servidor IMAP.
     * @param string    $folder             Nome da pasta que a rotina irá buscar os emails.
     * @param string    $imapOpenOptions    Opções de conexão do IMAP (veja todas as opções em 'http://php.net/manual/pt_BR/function.imap-open.php').
     * 
    */
    private function buildHostname($hostname, $folder = 'INBOX', $imapOpenOptions = '/notls'){
        $hostname = '{'.$hostname.$imapOpenOptions.'}'.$folder;
        return $hostname;
    }
    
    
    /**
     * Função para criar o diretório de Anexos
     * (necessário para que a biblioteca de IMAP leia os anexos).
     * 
    */
    private function checkAttachDir(){
        if(!is_dir($this->tempDirectoryPath)){
            mkdir($this->tempDirectoryPath);
        }
    }
    
    public function createOper(DTO\Request $row, DTO\Response $response){
        try {
            
            $return = $this->ticketBotService->createOperAndRedirect($row->getRow()[0]);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('OPER', is_array($return) ? $return : array($return)));

        } catch(\Exception $e) {
            
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    /**
     * Função para criar o job de criação de tickets 
     * no jenkins.
     * 
     * @param number    $nrorg      Número da organização.
     * 
    */
    public function createJobInJenkins($nrorg, $response){
        try{
            $jenkins = new Jenkins('http://zeedhi:1070ddda4a72a6a4bc47b047f1fe2681@jenkins.zeedhi.com');
            
            $config = file_get_contents(__DIR__.'/../Job/config.xml');
            $config = str_replace('%nrorg%', $nrorg, $config);
            
            $jenkins->createJob('WebmailTicketCRM_ORG_'.$nrorg, $config);
            
            $response->addDataSet(new DataSet('job', array('Job created!')));
        }catch(\Exception $e){
            //10->já existe o job
            if($e->getCode() === 10){
                $response->addDataSet(new DataSet('job', array('Job already exists!')));
            }else{
                $response->addDataSet(new DataSet('job', array('Problem with Jenkins!')));
            }
        }
        
    }
    
    /**
     * Função para deletar o job de criação de tickets 
     * no jenkins.
     * 
     * @param number    $nrorg      Número da organização.
     * 
    */
    public function deleteJobInJenkins($nrorg, $response){
        try{
            $jenkins = new Jenkins('http://zeedhi:1070ddda4a72a6a4bc47b047f1fe2681@jenkins.zeedhi.com');
            $jenkins->deleteJob('WebmailTicketCRM_ORG_'.$nrorg);
            
            $response->addDataSet(new DataSet('job', array('Job Deleted!')));
        }catch(\Exception $e){
            $response->addDataSet(new DataSet('job', array('Error deleting job.')));
            
        }
        
    }
}